#-------------------------------------------------------------------------------
# Name:        LineAdjacencyTable
# Purpose:     Determine adjacency relationships between lines
# Assumptions: Line features are already topologically correct
#
# Author:      Barry J Kronenfeld
#
# Created:     11/04/2015
# Copyright:   (c) Barry J Kronenfeld 2015
# Licence:     <your licence>
#-------------------------------------------------------------------------------

import time
workrecord=[]
starttime=time.time()
print "importing arcpy!!!"
import arcpy
arcpy.env.overwriteOutput=True
workrecord.append([time.time(),"importing arcpy"])

def reportMessage(msg):
    print msg
    arcpy.AddMessage(msg)

def lineAdjacencyTable(linefc):
    reportMessage("getting line feature adjacency table...")
    # Extract nodes
    reportMessage("extracting end nodes...")
    nodes = arcpy.FeatureVerticesToPoints_management(linefc,r'in_memory\nodes',point_location="BOTH_ENDS")
    numnodes=int(arcpy.GetCount_management(nodes).getOutput(0))
    reportMessage("found " + str(numnodes) + " nodes!")
    workrecord.append([time.time(),"extracting nodes"])
    # get line id field, max line id
    reportMessage("analyzing input...")
    desc = arcpy.Describe(linefc)
    lineidfield = desc.OIDFieldName
    reportMessage("ID field is " + lineidfield)
    sc = arcpy.da.SearchCursor(linefc,[lineidfield])
    maxlineid=-1
    for row in sc:
        if row[0]>maxlineid:
            maxlineid=row[0]
    del sc
    reportMessage("max object ID is " + str(maxlineid))
    workrecord.append([time.time(),"analyzing input"])
    # create list of original line feature IDs for each node
    reportMessage("indexing nodes to line features...")
    desc=arcpy.Describe(nodes)
    nodeidfield = desc.OIDFieldName
    sc = arcpy.da.SearchCursor(nodes,["ORIG_FID",nodeidfield])
    maxnodeid=-1
    for row in sc:
        if row[1]>maxnodeid:
            maxnodeid=row[1]
    node_lineid =[-1]*(maxnodeid+1)
    sc.reset()
    for row in sc:
        node_lineid[row[1]]=row[0]
    del sc
    workrecord.append([time.time(),"creating node-line index"])
    # find identical nodes
    reportMessage("finding coincident nodes...")
    arcpy.AddXY_management(nodes)
    # my way
    nodesdata=[]
    sc=arcpy.da.SearchCursor(nodes,[nodeidfield,"POINT_X", "POINT_Y"])
    for row in sc:
        # get information
        id=row[0]
        thisx=row[1]
        thisy=row[2]
        nodesdata.append([id,thisx,thisy])
    import operator
    nodesdata=sorted(nodesdata,key=operator.itemgetter(1))
    nodesdata=sorted(nodesdata,key=operator.itemgetter(2))
    adjgroups=[]
    lastx=None
    lasty=None
    curgroup=[]
    for nodedata in nodesdata:
        id=nodedata[0]
        thisx=nodedata[1]
        thisy=nodedata[2]
        sameGroup=False
        if thisx==lastx:
            if thisy==lasty:
                sameGroup=True
        if sameGroup==False:
            # process last group
            if len(curgroup)>1:
                adjgroups.append(curgroup)
            # start new group
            curgroup=[]
        # add node id to group
        curgroup.append(id)
        # update last xy
        lastx=thisx
        lasty=thisy
    # handle last group
    if len(curgroup)>1:
        adjgroups.append(curgroup)

    workrecord.append([time.time(),"finding coincident nodes"])
##    print "------------"
##    print "MY GROUPS:"
##    for row in adjgroups:
##        print row
##    print "------------"

    myadjacency =[[] for _ in range(maxlineid+1)]
    for row in adjgroups:
        lineids=[]
        for nodeid in row:
            lineids.append(node_lineid[nodeid])
        for id1 in lineids:
            for id2 in lineids:
                if id1<>id2:
                    myadjacency[id1].append(id2)
    workrecord.append([time.time(),"recording adjacency"])

##    coincident = arcpy.env.scratchGDB + r'\coincident'
##    arcpy.FindIdentical_management(nodes, coincident, 'POINT_X;POINT_Y', '#', '0', 'ONLY_DUPLICATES')
##    numIdentical= arcpy.GetCount_management(coincident).getOutput(0)
##    print "Found " + str(numIdentical) + " nodes"
##    workrecord.append([time.time(),"finding coincident nodes"])
##
##    # work through output table to extract adjacency relationships
##    reportMessage("Recording adjacency relationships")
##    fields=["IN_FID","FEAT_SEQ"]
##    sc = arcpy.da.SearchCursor(coincident,fields)
##    adjacency =[[] for _ in range(maxlineid+1)]
##    matchlinelist=[]
##    lastfeatseq=-1
##    # loop through rows in FindIdentical table result
##    for row in sc:
##        # extract values
##        curid=node_lineid[row[0]]
##        seq=row[1]
##        # if we're at a new sequence, process the last list and clear it
##        if seq<>lastfeatseq:
##            # process matchlinelist
##            for id1 in matchlinelist:
##                for id2 in matchlinelist:
##                    if id1<>id2:
##                        adjacency[id1].append(id2)
##            # clear list
##            matchlinelist=[]
##        # add feature to match line list
##        matchlinelist.append(curid)
##        # update last sequence
##        lastfeatseq=seq
##    # when finished the loop, there is still one more list of identical features to process
##    for id1 in matchlinelist:
##        for id2 in matchlinelist:
##            if id1<>id2:
##                adjacency[id1].append(id2)
##    workrecord.append([time.time(),"recording adjacency"])
##    # check the results
##    id=0
##    for adjlist in adjacency:
##        print id, adjlist
##        id+=1

    # report times
    reportMessage("-------------")
    reportMessage("TIME ANALYSIS:")
    lasttime=starttime
    for row in workrecord:
        thistime=row[0]
        elapsed=thistime-lasttime
        thisjob=row[1]
        reportMessage(thisjob + ": " + "{:10.4f}".format(elapsed))
        lasttime=thistime
    elapsed=lasttime-starttime
    print "TOTAL: {:10.4f} seconds".format(elapsed)

##    # test my results
##    id=0
##    for myrow in myadjacency:
##        print id, myrow
##        id+=1
##    id=0
##    for row in adjacency:
##        print id,row
##        id+=1

    # return result
    return myadjacency

if __name__ == '__main__':
    linefc = r'C:\sync\teaching\GIS_Modeling\Python\CoursePackage\SampleData\Strahler.gdb\Final_Network'
    lineAdjacencyTable(linefc)
    reportMessage("Finished!")