#-------------------------------------------------------------------------------
# Name:        concavePoints
# Purpose:     Finds concave points on the edge of each selected polygon in input
#              and saves to a new point feature class.
#              Mostly this is preprocessing for a medial axis tool
#
# Author:      Barry Kronenfeld
#
# Created:     20/03/2015
# Copyright:   (c) Barry Kronenfeld 2015
# Licence:     GPL v3.0
#-------------------------------------------------------------------------------

import arcpy
import arcpy.da


arcpy.env.overwriteOutput=True
arcpy.Delete_management("in_memory")

def reportMessage(msg):
    print msg
    arcpy.AddMessage(msg)

def polyarea(ptlist):
    # calculates area for a list of lists of xy values
    area=0
    n=len(ptlist)
    for id in range(len(ptlist)):
        nextid=(id+1)%n
        x0=ptlist[id][0]
        y0=ptlist[id][1]
        x1=ptlist[nextid][0]
        y1=ptlist[nextid][1]
        area += (x1-x0)*(y0+y1)/2
    return area

def getangle(ptlist):
    # calculates the angle of the middle vertex, in degrees
    # get vertices
    ax=ptlist[0][0]
    ay=ptlist[0][1]
    bx=ptlist[1][0]
    by=ptlist[1][1]
    cx=ptlist[2][0]
    cy=ptlist[2][1]
    # convert to vectors
    ax=ax-bx
    ay=ay-by
    cx=cx-bx
    cy=cy-by
    # get dot product
    dotproduct=ax*cx + ay*cy
    # get lengths
    alen = math.sqrt(ax*ax + ay*ay)
    clen = math.sqrt(cx*cx + cy*cy)
    deg=180
    if alen>0 and clen > 0:
        # get theta
        theta = math.acos((dotproduct)/(alen*clen))
        # convert to degrees
        deg = theta * 180 / math.pi
    # return result
    return deg

def concavePoints(inpolyfc, outws,outfc,maxangle,minarea):
    # create output feature class
    reportMessage("Creating output feature class...")
    resultfc=arcpy.CreateFeatureclass_management(outws,outfc,"POINT",spatial_reference=inpolyfc)
    arcpy.AddField_management(resultfc,"PolyID","LONG")
    ic = arcpy.da.InsertCursor(resultfc,["SHAPE@XY","PolyID"])
    # get search cursor
    reportMessage("Stepping through features...")
    sc = arcpy.da.SearchCursor(inpolyfc,["SHAPE@","OID@"])
    # step through shapes
    for row in sc:
        print "Shape " + str(row[1])
        partnum=0
        OID = row[1]
        # step through parts
        for part in row[0]:
            print("Part {0}:".format(partnum))
            # step through points to create point lists for each part
            ptlist=[[]]
            partnum=0
            for pnt in part:
                if pnt:
                    ptlist[partnum].append([pnt.X, pnt.Y])
                else:
                    ptlist.append([])
                    partnum+=1
            # step through rings
            for ring in ptlist:
                # step through points
                for ptid in range(len(ring)-1):
                    # get previous and next point ids
                    previd=(ptid-1)%len(ring)
                    nextid=ptid+1
                    # get triangle vertices
                    trivert=[ring[previd],ring[ptid],ring[nextid]]
                    print trivert
                    # calculate area of triangle
                    area=polyarea(trivert)
                    # calculate angle
                    angle = getangle(trivert)
                    # add point
                    if area>minarea:
                        if angle < maxangle:
                            ptgeo=(ring[ptid][0],ring[ptid][1])
                            ic.insertRow([ptgeo, OID])
    del sc
    del ic


if __name__ == '__main__':
    reportMessage("Initializing...")
##    inpolyfc=r'C:\sync\teaching\GIS_Modeling\Python\testarea.gdb\PolygonsWithHoles'
##    outws=r'C:\sync\teaching\GIS_Modeling\Python\testarea.gdb'
##    outptfc=r'ConcavePoints'

    inpolyfc=arcpy.GetParameterAsText(0)
    outws=arcpy.GetParameterAsText(1)
    outptfc=arcpy.GetParameter(2)
    maxangle=arcpy.GetParameter(3)
    minarea=arcpy.GetParameter(4)

    concavePoints(inpolyfc,outws,outptfc,maxangle,minarea)
    print "finished!"
