#-------------------------------------------------------------------------------
# Name:        module1
# Purpose:
#
# Author:      Ximing
#
# Created:     27/03/2015
# Copyright:   (c) Ximing 2015
# Licence:     <your licence>
#-------------------------------------------------------------------------------

print "importing arcpy..."
import arcpy
arcpy.env.overwriteOutput=True

def reportMessage(msg):
    print msg
    arcpy.AddMessage(msg)

def featid(fc):
    # obtain list of polyIDs
    numpolyobj = arcpy.GetCount_management(fc)
    numpoly = int(numpolyobj.getOutput(0))
    result =[0]*numpoly
    sc = arcpy.da.SearchCursor(fc, ["OID@"])
    curid = 0
    for row in sc:
        result[curid]=row[0]
        curid+=1
    del sc
    return result

def connectMedialAxisToEdgePoints(polys,mas,mapolyidfield,edgepts,edgeptpolyidfield, outfc):
    reportMessage("Initializing...")

    # obtain feature vertices
    reportMessage("Obtaining vertices...")
    print "--feature vertices to points (x2)"
    polyvert = arcpy.FeatureVerticesToPoints_management(polys,r'In_Memory\polyvert')
    mavert = arcpy.FeatureVerticesToPoints_management(mas,r'In_Memory\mavert')

    # merge into single layer
    print "--merge"
    allvert = arcpy.Merge_management([polyvert,mavert,edgepts],r'In_Memory\allvert_duplicates')

    # remove duplicates
    print "--addxy"
    arcpy.AddXY_management(allvert)
    print "==delete  identical"
    arcpy.DeleteIdentical_management(allvert,["POINT_X", "POINT_Y"])

    # create TIN edges
    reportMessage("creating TIN...")
    print "== check out extension"
    arcpy.CheckOutExtension("3D")
    infeat = r"In_Memory\allvert_duplicates POINT_X Mass_Points; " + polys + " SHAPE_Length Hard_Clip"
    tinname= arcpy.env.scratchFolder +  r'\basetin'
    print "-- create TIN"
    TIN = arcpy.CreateTin_3d(tinname, polys, infeat,"CONSTRAINED_DELAUNAY")
    print "-- tin edge"
    tinedges = arcpy.TinEdge_3d(TIN,r'In_Memory\tinedges',"ALL")
    print "-- delete tin and check in extension"
    del TIN
    arcpy.CheckInExtension("3D")

    # add object ID field onto edge points
    # first make copy of edge pts in scratch work space
    edgeptcopyfilename = arcpy.env.scratchGDB + r'\edgeptwid'
    print "-- copy features"
    edgeptwid = arcpy.CopyFeatures_management(edgepts,edgeptcopyfilename)
    arcpy.AddField_management(edgeptwid,"edgeptid","LONG")
    print "-- describe"
    oidfield = arcpy.Describe(edgeptwid).OIDFieldName
    calcexp = "!" + oidfield + "!"
    print "-- calculate field"
    arcpy.CalculateField_management(edgeptwid,"edgeptid",calcexp, "PYTHON_9.3")

    # select edges that go from matching MA to edge points
    reportMessage("Weeding edges...")
    print "-- make feature layer"
    tinedgelayer = arcpy.MakeFeatureLayer_management(tinedges,r'In_Memory\tinedgelayer')
    print "-- spatial join (x2)"
    edgestouchingedgepts= arcpy.SpatialJoin_analysis(tinedgelayer,mas,r'In_Memory\edgestouchingedgepts',join_type="KEEP_COMMON")
    edgestouchingboth = arcpy.SpatialJoin_analysis(edgestouchingedgepts,edgeptwid,r'In_Memory\edgestouchingboth',join_type="KEEP_COMMON")
    print "-- make feature layer"
    tinedgelayer=arcpy.MakeFeatureLayer_management(edgestouchingboth,r'In_Memory\tinedgelayer2')
    print "-- select by attribute"
    if edgeptpolyidfield==mapolyidfield:
        edgeptpolyidfield=edgeptpolyidfield + "_1"
    clause = mapolyidfield + " = " + edgeptpolyidfield
    potentialedges = arcpy.SelectLayerByAttribute_management(tinedgelayer,where_clause=clause)
    # select shortest edge for each
    # add length field
    fcname = arcpy.env.scratchGDB + r'\edgeswithlengths'
    #fcname = r'In_Memory\edgeswithlengths'
    print "-- copy features"
    edgeswithlengths = arcpy.CopyFeatures_management(potentialedges,fcname)
    arcpy.AddField_management(edgeswithlengths,"length_m","FLOAT")
    exp = "!SHAPE.LENGTH@METERS!"
    print "-- calculate field"
    arcpy.CalculateField_management(edgeswithlengths,"length_m",exp,"PYTHON_9.3")
    # loop through in order of edgeptid, then length
    uc = arcpy.da.UpdateCursor(edgeswithlengths,["edgeptid","length_m"], sql_clause = (None,"ORDER BY edgeptid, length_m"))
    lastID=None
    # since edges associated with each edgept are in ascending order of length,
    # we can remove all but the first edge associated with each edgept
    numdeleted=0
    numsaved=0
    for row in uc:
        thisID=row[0]
        if thisID==lastID:
            numdeleted+=1
            uc.deleteRow()
        else:
            numsaved+=1
            lastID=thisID
    print numdeleted,"deleted",numsaved,"saved"
    del uc
    # save results
    reportMessage("saving results...")
    arcpy.CopyFeatures_management(edgeswithlengths,outfc)
    reportMessage("finished successfully.")

if __name__ == '__main__':
##    polys = r'C:\sync\teaching\GIS_Modeling\Python\testarea.gdb\PolygonsWithHoles'
##    mas = r'C:\sync\teaching\GIS_Modeling\Python\testarea.gdb\ma5'
##    ma_polyidfield = "PolyID"
##    edgepts = r'C:\sync\teaching\GIS_Modeling\Python\testarea.gdb\concave2'
##    edgept_polyidfield = "PolyID"
##    outfc = r'C:\sync\teaching\GIS_Modeling\Python\testarea.gdb\edge_connections'

    polys=arcpy.GetParameterAsText(0)
    mas=arcpy.GetParameterAsText(1)
    ma_polyidfield=arcpy.GetParameterAsText(2)
    edgepts=arcpy.GetParameterAsText(3)
    edgept_polyidfield=arcpy.GetParameterAsText(4)
    outfc=arcpy.GetParameterAsText(5)

    connectMedialAxisToEdgePoints(polys,mas,ma_polyidfield,edgepts,edgept_polyidfield,outfc)
